Package: TDApplied
Type: Package
Title: Machine Learning and Inference for Topological Data Analysis
Version: 3.0.4
Authors@R: c(person("Shael", "Brown", email = "shaelebrown@gmail.com", role = c("aut","cre")),
             person("Dr. Reza", "Farivar", email = "reza.farivar@mcgill.ca", role = c("aut","fnd")))
Author: Shael Brown [aut, cre],
  Dr. Reza Farivar [aut, fnd]
Maintainer: Shael Brown <shaelebrown@gmail.com>
Description: Topological data analysis is a powerful tool for finding non-linear global structure
    in whole datasets. The main tool of topological data analysis is persistent homology, which computes
    a topological shape descriptor of a dataset called a persistence diagram. 'TDApplied' provides 
    useful and efficient methods for analyzing groups of persistence diagrams with machine learning and statistical inference,
    and these functions can also interface with other data science packages to form flexible and integrated
    topological data analysis pipelines.
Depends: R (>= 3.5.0)
Imports: parallel, doParallel, foreach, clue, rdist, parallelly,
        kernlab, iterators, methods, stats, utils, Rcpp (>= 0.11.0)
License: GPL (>= 3)
URL: https://github.com/shaelebrown/TDApplied
BugReports: https://github.com/shaelebrown/TDApplied/issues
Encoding: UTF-8
NeedsCompilation: yes
RoxygenNote: 7.3.2
Suggests: rmarkdown, knitr, testthat (>= 3.0.0), TDAstats, reticulate,
        TDA, igraph
LinkingTo: Rcpp
VignetteBuilder: knitr, rmarkdown
Config/testthat/edition: 3
Packaged: 2024-10-28 13:18:16 UTC; jibaccount
Repository: CRAN
Date/Publication: 2024-10-29 08:30:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 05:03:44 UTC; unix
Archs: TDApplied.so.dSYM
