# Generated by roxygen2: do not edit by hand

export(PyH)
export(bootstrap_persistence_thresholds)
export(diagram_distance)
export(diagram_kernel)
export(diagram_kkmeans)
export(diagram_kpca)
export(diagram_ksvm)
export(diagram_mds)
export(diagram_to_df)
export(distance_matrix)
export(enclosing_radius)
export(gram_matrix)
export(import_ripser)
export(independence_test)
export(permutation_model_inference)
export(permutation_test)
export(plot_diagram)
export(plot_vr_graph)
export(predict_diagram_kkmeans)
export(predict_diagram_kpca)
export(predict_diagram_ksvm)
export(universal_null)
export(vr_graphs)
import(Rcpp)
importFrom(clue,solve_LSAP)
importFrom(doParallel,registerDoParallel)
importFrom(doParallel,stopImplicitCluster)
importFrom(foreach,"%:%")
importFrom(foreach,"%do%")
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(iterators,iter)
importFrom(kernlab,as.kernelMatrix)
importFrom(kernlab,kkmeans)
importFrom(kernlab,kpca)
importFrom(kernlab,ksvm)
importFrom(kernlab,predict)
importFrom(methods,is)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(parallelly,availableCores)
importFrom(rdist,cdist)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,cmdscale)
importFrom(stats,complete.cases)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,heatmap)
importFrom(stats,order.dendrogram)
importFrom(stats,pgamma)
importFrom(stats,quantile)
importFrom(utils,combn)
useDynLib(TDApplied, .registration = TRUE)
